BASEURL = "https://welearn.iiserkol.ac.in"
LINK_CACHE = ".link_cache"
COURSE_CACHE = ".course_cache"
EVENT_CACHE = "~/.welearn_event_cache"
ARCHIVE_TYPES = [
    "7Z",
    "GZ",
    "GZIP",
    "TAR",
    "TGZ",
    "ZIP",
]
